@@ -1,6 +1,6 @@
 from extras.plugins import PluginConfig
 
-__version__ = "0.6.0"
+__version__ = "0.9.0"
 
 
 class DNSConfig(PluginConfig):
@@ -8,19 +8,21 @@ class DNSConfig(PluginConfig):
     verbose_name = "Netbox DNS"
     description = "Netbox DNS"
     min_version = "3.0.0"
-    max_version = None
+    max_version = "3.1.99"
     version = __version__
     author = "Aurora Research Lab"
     author_email = "info@aurorabilisim.com"
     required_settings = []
     default_settings = {
         "zone_default_ttl": 86400,
+        "zone_soa_serial_auto": True,
         "zone_soa_serial": 1,
         "zone_soa_refresh": 172800,
         "zone_soa_retry": 7200,
         "zone_soa_expire": 2592000,
         "zone_soa_minimum": 3600,
     }
+    base_url = "netbox-dns"
 
 
 config = DNSConfig
