@@ -3,21 +3,22 @@ from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.core.validators import (
     MinValueValidator,
+    MaxValueValidator,
     validate_ipv6_address,
     validate_ipv4_address,
 )
-from django.forms import CharField, IntegerField
+from django.forms import (
+    CharField,
+    IntegerField,
+    BooleanField,
+    NullBooleanField,
+)
 from django.urls import reverse_lazy
 
-from extras.forms import (
-    CustomFieldModelForm,
-    CustomFieldModelCSVForm,
-    AddRemoveTagsForm,
-    CustomFieldModelBulkEditForm,
-    CustomFieldModelFilterForm,
-)
+from extras.forms import AddRemoveTagsForm
 from extras.models.tags import Tag
 from utilities.forms import (
+    CSVModelForm,
     BootstrapMixin,
     BulkEditNullBooleanSelect,
     DynamicModelMultipleChoiceField,
@@ -34,25 +35,36 @@ from .fields import CustomDynamicModelMultipleChoiceField
 from .models import NameServer, Record, Zone
 
 
-class ZoneForm(BootstrapMixin, CustomFieldModelForm):
+class BulkEditForm(forms.Form):
+    """Base form for editing multiple objects in bulk."""
+
+    def __init__(self, model, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.model = model
+        self.nullable_fields = []
+
+        if hasattr(self.Meta, "nullable_fields"):
+            self.nullable_fields = self.Meta.nullable_fields
+
+
+class ZoneForm(BootstrapMixin, forms.ModelForm):
     """Form for creating a new Zone object."""
 
     def __init__(self, *args, **kwargs):
-        """Override the __init__ method in order to provide the initial value for the default_ttl field"""
+        """Override the __init__ method in order to provide the initial value for the default fields"""
         super().__init__(*args, **kwargs)
 
         defaults = settings.PLUGINS_CONFIG.get("netbox_dns")
 
         def _initialize(initial, setting):
-            if not initial.get(setting, None):
+            if initial.get(setting, None) in (None, ""):
                 initial[setting] = defaults.get(f"zone_{setting}", None)
 
         for setting in (
             "default_ttl",
             "soa_ttl",
-            "soa_mname",
             "soa_rname",
-            "soa_serial",
+            "soa_serial_auto",
             "soa_refresh",
             "soa_retry",
             "soa_expire",
@@ -63,6 +75,26 @@ class ZoneForm(BootstrapMixin, CustomFieldModelForm):
         if self.initial.get("soa_ttl", None) is None:
             self.initial["soa_ttl"] = self.initial.get("default_ttl", None)
 
+        if self.initial.get("soa_serial_auto"):
+            self.initial["soa_serial"] = None
+
+        if self.initial.get("soa_mname", None) in (None, ""):
+            default_soa_mname = defaults.get("zone_soa_mname", None)
+            if default_soa_mname is not None:
+                try:
+                    self.initial["soa_mname"] = NameServer.objects.get(
+                        name=default_soa_mname
+                    )
+                except NameServer.DoesNotExist:
+                    pass
+
+        if not self.initial.get("nameservers", []):
+            default_nameservers = defaults.get("zone_nameservers", [])
+            if default_nameservers:
+                self.initial["nameservers"] = NameServer.objects.filter(
+                    name__in=default_nameservers
+                )
+
     def clean_default_ttl(self):
         return (
             self.cleaned_data["default_ttl"]
@@ -95,8 +127,13 @@ class ZoneForm(BootstrapMixin, CustomFieldModelForm):
         label="SOA Responsible",
         help_text="Mailbox of the zone's administrator",
     )
+    soa_serial_auto = BooleanField(
+        required=False,
+        label="Generate SOA Serial",
+        help_text="Automatically generate the SOA Serial",
+    )
     soa_serial = IntegerField(
-        required=True,
+        required=False,
         label="SOA Serial",
         help_text="Serial number of the current zone data version",
         validators=[MinValueValidator(1)],
@@ -137,25 +174,32 @@ class ZoneForm(BootstrapMixin, CustomFieldModelForm):
             "soa_ttl",
             "soa_mname",
             "soa_rname",
+            "soa_serial_auto",
             "soa_serial",
             "soa_refresh",
             "soa_retry",
             "soa_expire",
             "soa_minimum",
         )
-
         widgets = {
             "status": StaticSelect(),
             "soa_mname": StaticSelect(),
         }
+        help_texts = {
+            "soa_mname": "Primary name server for the zone",
+        }
 
 
-class ZoneFilterForm(BootstrapMixin, CustomFieldModelFilterForm):
+class ZoneFilterForm(BootstrapMixin, forms.Form):
     """Form for filtering Zone instances."""
 
     model = Zone
 
-    q = CharField(required=False, label="Search")
+    q = CharField(
+        required=False,
+        widget=forms.TextInput(attrs={"placeholder": "Name or Status"}),
+        label="Search",
+    )
     status = forms.ChoiceField(
         choices=add_blank_choice(Zone.CHOICES),
         required=False,
@@ -172,7 +216,7 @@ class ZoneFilterForm(BootstrapMixin, CustomFieldModelFilterForm):
     tag = TagFilterField(Zone)
 
 
-class ZoneCSVForm(CustomFieldModelCSVForm):
+class ZoneCSVForm(CSVModelForm, BootstrapMixin, forms.ModelForm):
     status = CSVChoiceField(
         choices=Zone.CHOICES,
         help_text="Zone status",
@@ -199,6 +243,10 @@ class ZoneCSVForm(CustomFieldModelCSVForm):
         required=False,
         help_text="Mailbox of the zone's administrator",
     )
+    soa_serial_auto = BooleanField(
+        required=False,
+        help_text="Generate the SOA serial",
+    )
     soa_serial = IntegerField(
         required=False,
         help_text="Serial number of the current zone data version",
@@ -252,8 +300,25 @@ class ZoneCSVForm(CustomFieldModelCSVForm):
     def clean_soa_rname(self):
         return self._clean_field_with_defaults("soa_rname")
 
+    def clean_soa_serial_auto(self):
+        try:
+            return self._clean_field_with_defaults("soa_serial_auto")
+        except ValidationError:
+            if self.cleaned_data["soa_serial"] or self._get_default_value("soa_serial"):
+                return None
+            else:
+                raise
+
     def clean_soa_serial(self):
-        return self._clean_field_with_defaults("soa_serial")
+        try:
+            return self._clean_field_with_defaults("soa_serial")
+        except ValidationError:
+            if self.cleaned_data["soa_serial_auto"] or self._get_default_value(
+                "soa_serial_auto"
+            ):
+                return None
+            else:
+                raise
 
     def clean_soa_refresh(self):
         return self._clean_field_with_defaults("soa_refresh")
@@ -276,6 +341,7 @@ class ZoneCSVForm(CustomFieldModelCSVForm):
             "soa_ttl",
             "soa_mname",
             "soa_rname",
+            "soa_serial_auto",
             "soa_serial",
             "soa_refresh",
             "soa_retry",
@@ -284,7 +350,7 @@ class ZoneCSVForm(CustomFieldModelCSVForm):
         )
 
 
-class ZoneBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldModelBulkEditForm):
+class ZoneBulkEditForm(BootstrapMixin, AddRemoveTagsForm, BulkEditForm):
     pk = forms.ModelMultipleChoiceField(
         queryset=Zone.objects.all(),
         widget=forms.MultipleHiddenInput(),
@@ -308,14 +374,29 @@ class ZoneBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldModelBulkEd
         label="SOA TTL",
         validators=[MinValueValidator(1)],
     )
+    soa_mname = DynamicModelChoiceField(
+        queryset=NameServer.objects.all(),
+        required=False,
+        label="SOA Primary Nameserver",
+        widget=APISelect(
+            attrs={
+                "data-url": reverse_lazy("plugins-api:netbox_dns-api:nameserver-list")
+            }
+        ),
+    )
     soa_rname = CharField(
         required=False,
         label="SOA Responsible",
     )
+    soa_serial_auto = NullBooleanField(
+        required=False,
+        widget=BulkEditNullBooleanSelect(),
+        label="Generate SOA Serial",
+    )
     soa_serial = IntegerField(
         required=False,
         label="SOA Serial",
-        validators=[MinValueValidator(1)],
+        validators=[MinValueValidator(1), MaxValueValidator(4294967295)],
     )
     soa_refresh = IntegerField(
         required=False,
@@ -338,6 +419,14 @@ class ZoneBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldModelBulkEd
         validators=[MinValueValidator(1)],
     )
 
+    def clean(self):
+        """
+        If soa_serial_auto is True, set soa_serial to None.
+        """
+        cleaned_data = super().clean()
+        if cleaned_data.get("soa_serial_auto"):
+            cleaned_data["soa_serial"] = None
+
     class Meta:
         nullable_fields = []
 
@@ -350,6 +439,7 @@ class ZoneBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldModelBulkEd
             "tags",
             "soa_ttl",
             "soa_rname",
+            "soa_serial_auto",
             "soa_serial",
             "soa_refresh",
             "soa_retry",
@@ -374,15 +464,11 @@ class NameServerForm(BootstrapMixin, forms.ModelForm):
         fields = ("name", "tags")
 
 
-class NameServerFilterForm(BootstrapMixin, CustomFieldModelFilterForm):
+class NameServerFilterForm(BootstrapMixin, forms.Form):
     """Form for filtering NameServer instances."""
 
     model = NameServer
 
-    q = CharField(
-        required=False,
-        label="Search",
-    )
     name = CharField(
         required=False,
         label="Name",
@@ -390,12 +476,25 @@ class NameServerFilterForm(BootstrapMixin, CustomFieldModelFilterForm):
     tag = TagFilterField(NameServer)
 
 
-class NameServerCSVForm(CustomFieldModelCSVForm):
+class NameServerCSVForm(CSVModelForm, BootstrapMixin, forms.ModelForm):
     class Meta:
         model = NameServer
         fields = ("name",)
 
 
+class NameServerBulkEditForm(BootstrapMixin, AddRemoveTagsForm, BulkEditForm):
+    pk = forms.ModelMultipleChoiceField(
+        queryset=NameServer.objects.all(),
+        widget=forms.MultipleHiddenInput(),
+    )
+
+    class Meta:
+        nullable_fields = []
+
+        model = NameServer
+        fields = ("name", "tags")
+
+
 class RecordForm(BootstrapMixin, forms.ModelForm):
     """Form for creating a new Record object."""
 
@@ -448,7 +547,7 @@ class RecordForm(BootstrapMixin, forms.ModelForm):
         else:
             return self.cleaned_data["zone"].default_ttl
 
-    disable_ptr = forms.BooleanField(
+    disable_ptr = BooleanField(
         label="Disable PTR",
         required=False,
     )
@@ -472,13 +571,14 @@ class RecordForm(BootstrapMixin, forms.ModelForm):
         }
 
 
-class RecordFilterForm(BootstrapMixin, CustomFieldModelFilterForm):
+class RecordFilterForm(BootstrapMixin, forms.Form):
     """Form for filtering Record instances."""
 
     model = Record
 
     q = CharField(
         required=False,
+        widget=forms.TextInput(attrs={"placeholder": "Name, Zone or Value"}),
         label="Search",
     )
     type = forms.MultipleChoiceField(
@@ -494,10 +594,15 @@ class RecordFilterForm(BootstrapMixin, CustomFieldModelFilterForm):
         required=False,
         label="Value",
     )
+    zone_id = CustomDynamicModelMultipleChoiceField(
+        queryset=Zone.objects.all(),
+        required=False,
+        label="Zone",
+    )
     tag = TagFilterField(Record)
 
 
-class RecordCSVForm(CustomFieldModelCSVForm):
+class RecordCSVForm(CSVModelForm, BootstrapMixin, forms.ModelForm):
     zone = CSVModelChoiceField(
         queryset=Zone.objects.all(),
         to_field_name="name",
@@ -513,6 +618,11 @@ class RecordCSVForm(CustomFieldModelCSVForm):
         required=False,
         help_text="TTL",
     )
+    disable_ptr = forms.BooleanField(
+        required=False,
+        label="Disable PTR",
+        help_text="Disable generation of a PTR record",
+    )
 
     def clean(self):
         """
@@ -562,12 +672,10 @@ class RecordCSVForm(CustomFieldModelCSVForm):
 
     class Meta:
         model = Record
-        fields = ("zone", "type", "name", "value", "ttl")
+        fields = ("zone", "type", "name", "value", "ttl", "disable_ptr")
 
 
-class RecordBulkEditForm(
-    BootstrapMixin, AddRemoveTagsForm, CustomFieldModelBulkEditForm
-):
+class RecordBulkEditForm(BootstrapMixin, AddRemoveTagsForm, BulkEditForm):
     pk = forms.ModelMultipleChoiceField(
         queryset=Record.objects.all(), widget=forms.MultipleHiddenInput()
     )
@@ -578,7 +686,7 @@ class RecordBulkEditForm(
             attrs={"data-url": reverse_lazy("plugins-api:netbox_dns-api:zone-list")}
         ),
     )
-    disable_ptr = forms.NullBooleanField(
+    disable_ptr = NullBooleanField(
         required=False, widget=BulkEditNullBooleanSelect(), label="Disable PTR"
     )
     ttl = IntegerField(
